@@ -67,7 +67,6 @@ setMethod("updateObject", "DataFrame",
                         "is current.\n",
                         "[updateObject] Nothing to update.")
         }
-
         callNextMethod()
     }
 )
@@ -282,24 +281,33 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE,
     varnames[dotnames == ""] <- list(NULL)
     nrows <- ncols <- integer(length(varnames))
     for (i in seq_along(listData)) {
-      element <- try(as(listData[[i]], "DFrame"), silent = TRUE)
+      var <- listData[[i]]
+      element <- try(as(var, "DFrame"), silent = TRUE)
       if (inherits(element, "try-error"))
-        stop("cannot coerce class \"", class(listData[[i]])[1L],
-             "\" to a DataFrame")
+        stop("cannot coerce class \"", class(var)[1L], "\" to a DataFrame")
       nrows[i] <- nrow(element)
       ncols[i] <- ncol(element)
       varlist[[i]] <- element
-      if (is(listData[[i]], "AsIs")) {
-        listData[[i]] <- drop_AsIs(listData[[i]])
+      if (is(var, "AsIs")) {
+        listData[[i]] <- drop_AsIs(var)
       } else {
-        if ((length(dim(listData[[i]])) > 1L) || (ncol(element) > 1L) ||
-             is.list(listData[[i]]))
-          {
-            if (is.null(varnames[[i]]))
-              varnames[[i]] <- colnames(element)
-            else
-              varnames[[i]] <- paste(varnames[[i]], colnames(element), sep = ".")
-          }
+        ## The only reason we use suppressWarnings() here is to suppress the
+        ## deprecation warning we get at the moment (BioC 3.14) when calling
+        ## dim() on a DataFrameList derivative. Remove when the dim() method
+        ## for DataFrameList derivatives is gone (note that when this happens,
+        ## dim() will return NULL on a DataFrameList derivative).
+        var_dim <- suppressWarnings(dim(var))
+        var_dims <- try(dims(var), silent=TRUE)
+        if (inherits(var_dims, "try-error"))
+            var_dims <- NULL
+        if (ncol(element) > 1L || is.list(var) ||
+            length(var_dim) > 1L || length(var_dims) > 1L)
+        {
+          if (is.null(varnames[[i]]))
+            varnames[[i]] <- colnames(element)
+          else
+            varnames[[i]] <- paste(varnames[[i]], colnames(element), sep = ".")
+        }
       }
       if (is.null(varnames[[i]])) {
           varnames[[i]] <- deparse(qargs[[i]])[1L]
@@ -307,7 +315,7 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE,
       if (missing(row.names))
         row.names <- rownames(element)
     }
-    mcols <- do.call(rbind_mcols, varlist)
+    mcols <- combine_mcols(varlist)
     varlist <- lapply(varlist, as.list, use.names = FALSE)
     nr <- max(nrows)
     for (i in which((nrows > 0L) & (nrows < nr) & (nr %% nrows == 0L))) {
@@ -404,7 +412,7 @@ setReplaceMethod("[[", "DataFrame",
 setMethod("extractROWS", "DataFrame",
     function(x, i)
     {
-        i <- normalizeSingleBracketSubscript(i, x, exact=FALSE, allow.NAs=TRUE,
+        i <- normalizeSingleBracketSubscript(i, x, allow.NAs=TRUE,
                                              as.NSBS=TRUE)
         slot(x, "listData", check=FALSE) <- lapply(as.list(x), extractROWS, i)
         slot(x, "nrows", check=FALSE) <- length(i)
@@ -869,7 +877,8 @@ setMethod("classNameForDisplay", "DFrame",
 setMethod("makeNakedCharacterMatrixForDisplay", "DataFrame",
     function(x)
     {
-        df <- as.data.frame(lapply(x, showAsCell), optional=TRUE)
+        df <- data.frame(lapply(x, showAsCell), check.names=FALSE,
+                         row.names=NULL)
         as.matrix(format(df))
     }
 )
@@ -925,220 +934,3 @@ setMethod("show", "DataFrame",
 
 setMethod("showAsCell", "DataFrame", showAsCell_array)
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining
-###
-
-.format_mismatch_message <- function(x, y) {
-    universe <- union(x, y)
-    tx <- table(factor(x, levels=universe))
-    ty <- table(factor(y, levels=universe))
-    common <- pmin(tx, ty)
-
-    leftx <- tx - common
-    lefty <- ty - common
-    misleft <- names(leftx)[leftx > 0]
-    misright <- names(lefty)[lefty > 0]
-
-    .format_failed_colnames <- function(x) {
-        x <- sprintf("'%s'", x)
-        if (length(x)>3) x <- c(head(x, 3), "...")
-        paste(x, collapse=", ")
-    }
-
-    if (length(misleft) && length(misright)) {
-        msg <- paste0("(", .format_failed_colnames(misleft),
-            " vs ", .format_failed_colnames(misright), ")")
-    } else if (length(misleft)) {
-        msg <- paste0("(", .format_failed_colnames(misleft), " ",
-            if (length(misleft) > 1) "are" else "is",
-            " unique)")
-    } else {
-        msg <- paste0("(", .format_failed_colnames(misright), " ",
-            if (length(misright) > 1) "are" else "is",
-            " unique)")
-    }
-
-    stop(wmsg("the DataFrame objects to rbind do not have ",
-              "the same column names ", msg))
-}
-
-### Return an integer matrix with 1 column per object in 'objects' and 1 row
-### per column in 'x'.
-.make_colmaps <- function(x, objects)
-{
-    x_colnames <- colnames(x)
-    x_ncol <- length(x_colnames)
-    map_x_colnames_to_object_colnames <- function(object_colnames) {
-        if (length(object_colnames) != x_ncol) {
-            .format_mismatch_message(x_colnames, object_colnames)
-        }
-        colmap <- selectHits(findMatches(x_colnames, object_colnames),
-                             select="first", nodup=TRUE)
-        if (anyNA(colmap)) {
-            .format_mismatch_message(x_colnames, object_colnames)
-        }
-        colmap
-    }
-    if (length(objects) == 0L) {
-        colmaps <- integer(0)
-    } else {
-        colmaps <- lapply(objects,
-            function(object)
-                map_x_colnames_to_object_colnames(colnames(object)))
-        colmaps <- unlist(colmaps, use.names=FALSE)
-    }
-    dim(colmaps) <- c(x_ncol, length(objects))
-    colmaps
-}
-
-### A thin wrapper around bindROWS()
-###
-### If all the columns to bind have the same type, the result of the binding
-### should be a column of that type (endomorphism). Note that this is what
-### happens with ordinary data frames. So we could just use bindROWS() for
-### that. However, when the columns to bind have mixed types, bindROWS()
-### won't necessarily do the right thing.
-### The purpose of the wrapper below is to improve handling of mixed type
-### columns by pre-processing some of them before calling bindROWS() on them.
-### More precisely, it tries to work around the 2 following problems that
-### direct use of bindROWS() on mixed type columns would pose:
-###  1) When the columns to bind are a mix of Rle and non-Rle objects,
-###     the type of the column returned by bindROWS() depends on the type
-###     of cols[[1]]. More precisely it's an Rle if and only if cols[[1]]
-###     is an Rle. The wrapper below **mitigate** this by decoding the Rle
-###     columns first. Note that this is a mitigation process only. For
-###     example it will help if Rle columns are mixed with atomic vectors
-###     or factors, but it won't help if cols[[1]] is an Rle and the other
-###     columns are IntegerList objects.
-###  2) When the columns to bind are a mix of atomic vectors and factors,
-###     bindROWS() would **always** return an atomic vector (whatever
-###     cols[[1]] is, i.e. atomic vector or factor). However we **always**
-###     want a factor. This is an intended deviation with respect to what
-###     rbind() does on ordinary data frames where the 1st data frame
-###     involved in the binding operation governs (i.e. a column in the
-###     result will be atomic vector or factor depending on what the
-###     corresponding column in the 1st data frame is).
-.bind_cols_along_their_ROWS <- function(cols)
-{
-    is_Rle <- vapply(cols, is, logical(1L), "Rle")
-    if (any(is_Rle) && !all(is_Rle))
-        cols[is_Rle] <- lapply(cols[is_Rle], decodeRle)
-    is_factor <- vapply(cols, is.factor, logical(1L))
-    if (any(is_factor)) {
-        cols <- lapply(cols, as.factor)
-        all_levels <- unique(unlist(lapply(cols, levels), use.names=FALSE))
-        cols <- lapply(cols, factor, all_levels)
-    }
-    bindROWS(cols[[1L]], cols[-1L])
-}
-
-### Ignore the 'ignore.mcols' argument!
-.bindROWS_DataFrame_objects <-
-    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
-{
-    if (!isTRUEorFALSE(use.names))
-        stop("'use.names' must be TRUE or FALSE")
-    objects <- prepare_objects_to_bind(x, objects)
-    all_objects <- c(list(x), objects)
-    has_rows <- vapply(all_objects, nrow, integer(1L), USE.NAMES=FALSE) > 0L
-    has_cols <- vapply(all_objects, ncol, integer(1L), USE.NAMES=FALSE) > 0L
-    if (!any(has_rows)) {
-        if (!any(has_cols))
-            return(x)
-        return(all_objects[[which(has_cols)[[1L]]]])
-    }
-    all_objects <- all_objects[has_rows]
-
-    ## From now on, all the objects to bind have rows.
-    x <- all_objects[[1L]]
-    objects <- all_objects[-1L]
-    colmaps <- .make_colmaps(x, objects)
-    if (ncol(x) == 0L) {
-        ans_listData <- x@listData
-        ans_nrow <- sum(unlist(lapply(all_objects, nrow), use.names=FALSE))
-    } else {
-        ans_listData <- lapply(setNames(seq_along(x), colnames(x)),
-            function(i) {
-                 x_col <- x[[i]]
-                 other_cols <- lapply(seq_along(objects),
-                                      function(j) objects[[j]][[colmaps[i, j]]])
-                 tryCatch(
-                     .bind_cols_along_their_ROWS(c(list(x_col), other_cols)),
-                     error=function(err) {
-                        stop("failed to rbind column '", colnames(x)[i],
-                            "' across DataFrame objects:\n  ",
-                            conditionMessage(err))
-                     }
-                 )
-            }
-        )
-        ans_nrow <- NROW(ans_listData[[1L]])
-    }
-    if (use.names) {
-        ## Bind the rownames.
-        ans_rownames <- unlist(lapply(all_objects, rownames), use.names=FALSE)
-        if (!is.null(ans_rownames)) {
-            if (length(ans_rownames) != ans_nrow) {
-                ## What we do here is surprising and inconsistent with
-                ## ordinary data frames.
-                ## TODO: Maybe reconsider this?
-                ans_rownames <- NULL  # why?
-            }
-        }
-    } else {
-        ans_rownames <- NULL
-    }
-    BiocGenerics:::replaceSlots(x, listData=ans_listData,
-                                   nrows=ans_nrow,
-                                   rownames=ans_rownames,
-                                   check=check)
-}
-
-### Defining bindROWS() gives us rbind().
-setMethod("bindROWS", "DataFrame", .bindROWS_DataFrame_objects)
-
-### S3/S4 combo for cbind.DataFrame
-cbind.DataFrame <- function(..., deparse.level=1)
-{
-    if (!identical(deparse.level, 1))
-        warning(wmsg("the cbind() method for DataFrame objects ",
-                     "ignores the 'deparse.level' argument"))
-    ## It's important that the call to DataFrame() below is able to deparse
-    ## the arguments in ... so for example
-    ##   b <- 11:13
-    ##   selectMethod("cbind", "DataFrame")(b)
-    ## returns a DataFrame with a column named "b".
-    ## This prevents us from calling DataFrame() via do.call() e.g. we can't
-    ## do something like
-    ##   objects <- delete_NULLs(list(...))
-    ##   do.call(DataFrame, c(objects, list(check.names=FALSE)))
-    ## because then DataFrame() wouldn't be able to deparse what was in ...
-    ## and selectMethod("cbind", "DataFrame")(b) would produce a DataFrame
-    ## with a column named "11:13".
-    DataFrame(..., check.names=FALSE)
-}
-setMethod("cbind", "DataFrame", cbind.DataFrame)
-
-### If we didn't define this method, calling c() on DataFrame objects would
-### call the "c" method for Vector objects, which just delegates to bindROWS()
-### so the binding would happen along the rows. This is not what we want so we
-### overwrite the "c" method for Vector objects with a method that binds along
-### the columns.
-setMethod("c", "DataFrame",
-    function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
-    {
-        if (!isTRUEorFALSE(ignore.mcols))
-            stop("'ignore.mcols' must be TRUE or FALSE")
-        if (!identical(recursive, FALSE))
-            stop(wmsg("\"c\" method for DataFrame objects ",
-                      "does not support the 'recursive' argument"))
-        objects <- unname(delete_NULLs(list(x, ...)))
-        ans <- do.call(cbind, objects)
-        if (ignore.mcols)
-            mcols(ans) <- NULL
-        ans
-    }
-)
-
